package environment

import error.ErrorManager

import java.io.InputStream
import java.io.PrintStream
import java.util.function.Function

object RuntimeEnvironment {

    var inputName: String? = null
    var input: InputStream? = null
    var output: PrintStream? = null
    var errorOutput: PrintStream? = null
    var errorManager: ErrorManager? = null
    var path: String? = null

    // TODO - convert to function types ()-> Unit etc... once calling code is kotlin
    var terminationFunction: Runnable? = null
    var errorTerminationFunction: Runnable? = null
    var promptDecorator: Function<String, String>? = null
    var valueOutputDecorator: Function<String, String>? = null
    var warningOutputDecorator: Function<String, String>? = null
    var errorOutputDecorator: Function<String, String>? = null
    var criticalOutputDecorator: Function<String, String>? = null

    fun reset() {
        inputName = null
        input = null
        output = null
        errorOutput = null
        errorManager = null
        path = null
        terminationFunction = null
        errorTerminationFunction = null
        promptDecorator = null
        valueOutputDecorator = null
        warningOutputDecorator = null
        errorOutputDecorator = null
        criticalOutputDecorator = null
    }

    fun terminateSuccessfully() {
        terminationFunction!!.run()
    }

    fun terminateExceptionally() {
        errorTerminationFunction!!.run()
    }

    fun decoratePrompt(prompt: String) = promptDecorator!!.apply(prompt)
    fun decorateValueOutput(valueOutput: String) = valueOutputDecorator!!.apply(valueOutput)
    fun decorateWarningOutput(warningOutput: String) = warningOutputDecorator!!.apply(warningOutput)
    fun decorateErrorOutput(errorOutput: String) = errorOutputDecorator!!.apply(errorOutput)
    fun decorateCriticalOutput(criticalOutput: String) = criticalOutputDecorator!!.apply(criticalOutput)
}
